-----------------------------------------------------------------
-- Alloy4DP -- Alloy for Declarative Processes
-- Copyright (C) 2012, 2013  Yoann Laurent (yoann.laurent@lip6.fr)

-- This program is free software: you can redistribute it and/or
-- modify it under the terms of the GNU General Public License as
-- published by the Free Software Foundation, either version 3 of the
-- License, or (at your option) any later version.

-- This program is distributed in the hope that it will be useful, but
-- WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-- General Public License for more details.

-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see
-- <http://www.gnu.org/licenses/>.
-----------------------------------------------------------------

// taken from "DecSerFlow: Towards a truly declarative service flow language."

open declarative

// ---- (1) [Process] 

one sig receive extends Activity {}
one sig hotel extends Activity {}
one sig compensation extends Activity {}
one sig airline extends Activity {}
one sig failedHotel extends Activity {}
one sig failedAirline extends Activity {}
one sig notifyFailure extends Activity {}
one sig bookedAirline extends Activity {}
one sig bookedHotel extends Activity {}
one sig notifyBooked extends Activity {}
one sig creditCard extends Activity {}

fact constraints {
	atomic[Activity]

	existence[receive, 1,1]
	existence[notifyFailure, 0,1]
	existence[notifyBooked, 0,1]
	existence[creditCard, 0,1]

	precedence[hotel, receive]
	precedence[airline, receive]
	(response[receive, hotel] or response[receive, airline])

	existence[compensation, 0, 1]
	negation_response[compensation, hotel]
	negation_response[compensation, airline]

	precedence[failedHotel, hotel]
	precedence[bookedHotel, hotel]

	(response[hotel, bookedHotel] or response[hotel, failedHotel])

	(precedence[compensation, failedHotel] or precedence[compensation, failedAirline])
	
	precedence[notifyFailure, compensation]

	(precedence[creditCard, bookedHotel] or precedence[creditCard, bookedAirline])

	precedence[bookedAirline, airline]
	
	precedence[failedAirline, airline]

	(response[airline, failedAirline] or response[airline, bookedAirline])

	negation_response[creditCard, hotel]
	negation_response[creditCard, airline]

	succession[creditCard, notifyBooked]

	negation_co_existence[notifyFailure, creditCard]

	one_existence[notifyFailure+creditCard] // mutual substitution
}

// ---- (2) [Goals]
pred goals1 {}
pred goals2 {
	existence[notifyBooked, 1, integer/max]
}
pred goals3 { 
	existence[notifyBooked,1,1] and response[notifyBooked, notifyFailure]
}
pred goals4 { 
	existence[notifyBooked, 0,0] and existence[notifyFailure, 0,0]
}
pred goals5 { 
	response[failedHotel, bookedHotel]
}
pred goals6 { 
	existence[notifyBooked,1,1]
}

// ---- (3) [Trace]
pred traces1 {
	traces[State0]
}
pred traces2 {
	setState[State1, receive]
	setState[State2, none]
	setState[State3, hotel]
	setState[State4, none]
	setState[State5, failedHotel]
	setState[State6, none]
	setState[State7, compensation]
	traces[State7]
}

// ---- [Alloy command] 
run {traces1 and goals1} for 0 but 20 State
run {traces1 and goals2} for 0 but 20 State
run {traces1 and goals3} for 0 but 20 State
run {traces1 and goals4} for 0 but 20 State
run {traces1 and goals5} for 0 but 20 State
run {traces2 and goals6} for 0 but 20 State


