-----------------------------------------------------------------
-- Alloy4DP -- Alloy for Declarative Processes
-- Copyright (C) 2012, 2013  Yoann Laurent (yoann.laurent@lip6.fr)

-- This program is free software: you can redistribute it and/or
-- modify it under the terms of the GNU General Public License as
-- published by the Free Software Foundation, either version 3 of the
-- License, or (at your option) any later version.

-- This program is distributed in the hope that it will be useful, but
-- WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-- General Public License for more details.

-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see
-- <http://www.gnu.org/licenses/>.
-----------------------------------------------------------------

// inspired from "Deviation Management during Process Execution"

open declarative

// ---- (1) [Process]  

one sig DesignModel extends Artifact {}
one sig ReviewDesignFeedback extends Artifact {}
one sig SourceCode extends Artifact {}
one sig TestFeedback extends Artifact {}

one sig ModifyDesign extends Activity {}
one sig ReviewDesign extends Activity {}
one sig ModifyCode extends Activity {}
one sig TestUnit extends Activity {}
one sig End extends Activity {}

one sig TestServer extends Resource {}

one sig InfoIPSW6 extends Info {} {
	use_resource_i = TestUnit -> TestServer 
}

fact constraints {

	// control-flow 
	atomic[Activity]

	initial[ModifyDesign]
	existence[ModifyDesign]
	chain_response[ModifyDesign, ReviewDesign]
	
	existence[ReviewDesign]
	alternate_precedence[ReviewDesign,ModifyDesign]
	(alternate_precedence[ModifyCode,ReviewDesign] or alternate_precedence[ModifyDesign,ReviewDesign])

	existence[ModifyCode]
	chain_response[ModifyCode, TestUnit]

	existence[TestUnit]
	(alternate_precedence[ModifyCode, TestUnit] or alternate_precedence[End, TestUnit])

	negation_response[TestUnit, ModifyDesign]
	negation_response[ModifyCode, ModifyDesign]
	
	existence[End,1,1]
	final[End]

	// data-flow
	all_data_in_out[ModifyDesign, none,DesignModel]
	all_data_in_out[ReviewDesign, DesignModel,ReviewDesignFeedback]
	all_data_in_out[ModifyCode, DesignModel,SourceCode]
	all_data_in_out[TestUnit, SourceCode,TestFeedback]
	all_data_in_out[End, none,none]

	// resources
	limited_use_resource[TestServer, 1]

	// time
	t_duration[ModifyDesign,4]	
	t_duration[ReviewDesign,1,3]	
	t_duration[ModifyCode,6]	
	t_duration[TestUnit,1,2]		
	t_existence[End,minTime,24]
}


// ---- (2) [Goals]
pred goals1 { 
}
// doable in 10 unit time?
pred goals2 { 
		t_existence[End,0,10]
}
// the sourcecode will be available before 10 times unit?
pred goals3 { 
		t_data_available[SourceCode,10]
}
// the sourcecode will be available before 14 times unit?
pred goals4 { 
		t_data_available[SourceCode,14]
}

// ---- (3) [Trace]
pred traces1 {
	traces[State0]
}

// ---- [Alloy command] 
run {traces1 and goals1} for 0 but 20 State ,6 Int expect 1
run {traces1 and goals2} for 0 but 20 State ,6 Int expect 0
run {traces1 and goals3} for 0 but 20 State ,6 Int expect 0
run {traces1 and goals4} for 0 but 20 State ,6 Int expect 1

